/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#ifndef __NET_WORLD_MODEL_H__
#define __NET_WORLD_MODEL_H__

#include <map>
#include <vector>

#include "net-world-node.h"
#include "net-world-time.h"

namespace ns3 {

  typedef std::map<uint32_t, NetWorldNode *> NodeIDMap;
  typedef std::map<std::string, NetWorldNode *> NodeIPMap;

/*
  class NetWorldSnapshot {
  public:
    NetWorldSnapshot();
    ~NetWorldSnapshot();

    NodeIDMap::iterator NodesBegin();
    NodeIDMap::iterator NodesEnd();

    bool AddNode(uint32_t id);

  private:
    NodeIDMap m_nodes;

    // end class NetWorldSnapshot
  };

  class NetWorldSnapshotHolder {
  public:
    NetWorldSnapshotHolder(NetWorldTime t);
    NetWorldSnapshotHolder(NetWorldTime t, NetWorldSnapshot *snapshot);
    ~NetWorldSnapshotHolder();

    NetWorldSnapshot *GetSnapshot();

  private:
    NetWorldTime m_timepoint;
    bool m_ownSnapshot;
    NetWorldSnapshot *m_snapshot;
  };

  typedef std::vector<NetWorldSnapshotHolder *> NetWorldSnapshotVector;

*/

  class NetWorldModel;

  class NetWorldEvent {
  public:
    NetWorldEvent(NetWorldTime t);
    virtual ~NetWorldEvent();

    NetWorldTime GetTimestamp();

    virtual void Process(NetWorldModel *worldModel) = 0;

  private:
    NetWorldTime m_timestamp;

    // end class NetWorldEvent
  };

  //------------------------------------------------------------

  class NetWorldIPEvent : public NetWorldEvent {
  public:
    NetWorldIPEvent(NetWorldTime t, uint32_t id, std::string ip);

    void Process(NetWorldModel *worldModel);

  private:
    uint32_t m_node;
    std::string m_ip;

    // end class NetWorldIPEvent
  };

  class NetWorldPositionEvent : public NetWorldEvent {
  public:
    NetWorldPositionEvent(NetWorldTime t, uint32_t id,
                          double x, double y, double z,
                          double xv, double yv, double zv);

    void Process(NetWorldModel *worldModel);

  private:
    uint32_t m_node;
    double m_x, m_y, m_z;
    double m_xvel, m_yvel, m_zvel;

    // end class NetWorldIPEvent
  };


  class NetWorldLinkStateEvent : public NetWorldEvent {
  public:
    NetWorldLinkStateEvent(NetWorldTime t, uint32_t id, std::string target,
                           bool link, std::string state);

    void Process(NetWorldModel *worldModel);

  private:
    uint32_t m_node;
    std::string m_target;
    bool m_link;
    std::string m_state;

    // end class NetWorldLinkStateEvent
  };

  class NetWorldNodeStateEvent : public NetWorldEvent {
  public:
    NetWorldNodeStateEvent(NetWorldTime t, uint32_t id,
                           bool add, std::string state);

    void Process(NetWorldModel *worldModel);

  private:
    uint32_t m_node;
    bool m_add;
    std::string m_state;

    // end class NetWorldNodeStateEvent
  };

  //------------------------------------------------------------

  typedef std::vector<NetWorldEvent *> NetWorldEventVector;

  class NetWorldModel {
  public:
    NetWorldModel();
    ~NetWorldModel();

    void SetArenaBounds(double x1, double y1, double x2, double y2);

    double GetLeft();
    double GetRight();
    double GetTop();
    double GetBottom();

    void PushEvent(NetWorldEvent *e);

    NodeIDMap::iterator NodesBegin();
    NodeIDMap::iterator NodesEnd();
    // bool AddNode(uint32_t id);

    void Start();
    bool End();
    void ProcessUpTo(NetWorldTime t);
    NetWorldTime GetLastTimestamp();

    NetWorldNode *GetNode(uint32_t id);
    NetWorldNode *GetNodeByIP(std::string ip);

    void MapIP(std::string ip, uint32_t nodeId);
    void SetPosition(uint32_t nodeId, NetWorldTime time,
                     double x, double y, double z,
                     double xv, double yv, double zv);

    void AddLinkState(uint32_t nodeID, std::string target, std::string state);
    void RemoveLinkState(uint32_t nodeID,std::string target,std::string state);

    void AddNodeState(uint32_t nodeID, std::string state);
    void RemoveNodeState(uint32_t nodeID, std::string state);

    void ClearNodes();

  private:

    double m_leftx, m_bottomy, m_rightx, m_topy;

    NodeIDMap m_nodes;

    NetWorldEventVector m_events;

    NetWorldEventVector::iterator m_currentEvent;

    NodeIPMap m_ips;

    // end class NetWorldModel
  };

  // end namespace ns3
};

#endif  // __NET_WORLD_MODEL_H__
