/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#ifndef __NET_WORLD_NODE_H__
#define __NET_WORLD_NODE_H__

#include <stdint.h>

#include <vector>
#include <map>
#include <set>

#include "net-world-time.h"

namespace ns3 {

  typedef std::set<std::string> LinkStateSet;
  typedef std::map<std::string, LinkStateSet > LinkMap;

  typedef std::set<std::string> NodeStateSet;

  class NetWorldNode {
  public:
    NetWorldNode(uint32_t id);

    void GetPosition(bool &hasPosition,
                     double &x, double &y, NetWorldTime tstamp) const;
    void SetPosition(double x, double y, NetWorldTime tstamp,
		     double m_xvel = 0, double m_yvel = 0);

    void AddIP(std::string ip);

    uint32_t GetID() const;

    void AddLinkState(std::string ip, std::string state);
    void RemoveLinkState(std::string ip, std::string state);

    void AddNodeState(std::string state);
    void RemoveNodeState(std::string state);

    LinkMap::iterator LinksBegin();
    LinkMap::iterator LinksEnd();

    NodeStateSet::iterator StatesBegin();
    NodeStateSet::iterator StatesEnd();

  private:
    uint32_t m_id;

    bool m_hasPosition;
    double m_x, m_y;
    double m_xvel, m_yvel;
    NetWorldTime m_positionTimestamp;

    std::vector<std::string> m_ips;

    LinkMap m_links;

    NodeStateSet m_states;

    // end class NetWorldNode
  };

  // end namespace ns3
};

#endif // __NET_WORLD_NODE_H__
