/* -*-  Mode: C++; c-file-style: "gnu"; indent-tabs-mode:nil; -*- */
/*
 * Copyright (c) 2009 Drexel University
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Author: Joe Kopena <tjkopena@cs.drexel.edu>
 */

#include "net-debug.h"

#include <gtkmm/scrolledwindow.h>
#include <gtkmm/main.h>
#include <gtkmm/window.h>
#include <gtkmm.h>

#include "animation-model.h"

using namespace ns3;

AnimationModel::AnimationModel() :
  m_currTime(0),
  m_model(0),
  m_interval(100),
  m_timeoutDuration(10),
  m_loop(true),
  m_playing(false)
{
  // end AnimationModel::AnimationModel
}

bool
AnimationModel::GetPlaying() const
{
  return m_playing;
  // end AnimationModel::GetPlaying
}

void
AnimationModel::ToggleLoop()
{
  m_loop = !m_loop;
  // end AnimationModel::ToggleLoop
}
bool
AnimationModel::GetLoop() const
{
  return m_loop;
  // end AnimationModel::GetLoop
}

void
AnimationModel::AddNotifier(AnimationNotifier *notifier)
{
  m_notifiers.push_back(notifier);
  // end AnimationModel::AddNotifier
}
void
AnimationModel::SetWorldModel(NetWorldModel *model)
{
  m_model = model;
  // end AnimationModel::SetWorldModel
}
void
AnimationModel::SetInterval(NetWorldTime interval)
{
  m_interval = interval;
  // end AnimationModel::SetInterval
}

void
AnimationModel::ResetTo(NetWorldTime t)
{

  m_currTime = t;
  m_model->Start();
  m_model->ProcessUpTo(t);

  Notify(t);
  // end AnimationModel::ResetTo
}

void
AnimationModel::Notify(NetWorldTime t)
{

  for (AnimationNotifierList::iterator i = m_notifiers.begin();
       i != m_notifiers.end();
       i++) {

    (*i)->SetAnimationTime(t);

    // end for notifiers
  }

  // end AnimationModel::Notify
}

bool
AnimationModel::Advance()
{

  bool res = true;

  m_currTime += m_interval;

  if (m_currTime > m_model->GetLastTimestamp()) {
    if (m_loop) {
      ResetTo(0);
    } else {
      m_currTime = m_model->GetLastTimestamp();
      m_model->ProcessUpTo(m_currTime);
      Notify(m_currTime);
      Stop();
      res = false;
      // end no loop
    }

    // end went beyond the event list
  } else {
      m_model->ProcessUpTo(m_currTime);
      Notify(m_currTime);
  }

  return res;

  // and AnimationModel::Advance
}

void
AnimationModel::Start()
{
  NET_LOG_NOARGS();

  sigc::slot<bool> tslot = sigc::mem_fun(*this,
					 &AnimationModel::Advance);

  // This is where we connect the slot to the Glib::signal_timeout()
  m_animTimeout = Glib::signal_timeout().connect(tslot, m_timeoutDuration);

  m_playing = true;

  // end AnimationModel::Start
}
void
AnimationModel::Stop()
{
  NET_LOG_NOARGS();
  m_animTimeout.disconnect();
  m_playing = false;
  // end AnimationModel::Stop
}
